#!/bin/bash


if ! command -v nasm &> /dev/null; then
    echo "Error: NASM (Netwide Assembler) is not installed or not in your PATH."
    echo "Please install NASM before running this script."
    exit 1
fi

mkdir -p build
nasm -f bin ./src/boot/stage1.asm -o ./build/stage1.bin 
nasm -f bin ./src/boot/stage2.asm -o ./build/stage2.bin 
nasm -f bin ./src/kernel/kernel.asm -o ./build/kernel.bin 
dd if=/dev/zero of=./build/ExploreOS-0.45.0028.img bs=512 count=2880
dd if=./build/stage1.bin of=./build/ExploreOS-0.45.0028.img bs=512 count=1 conv=notrunc 
dd if=./build/stage2.bin of=./build/ExploreOS-0.45.0028.img bs=512 seek=1 conv=notrunc 
dd if=./build/kernel.bin of=./build/ExploreOS-0.45.0028.img bs=512 seek=3 conv=notrunc
rm ./build/*.bin

echo "Build complete! Disk image created at ./build/ExploreOS-0.45.0028.img"
echo "Would you like to test it in QEMU? (y/n)"
read answer
if [ "$answer" = "y" ] || [ "$answer" = "Y" ]; then
    if command -v qemu-system-i386 &> /dev/null; then
        echo "Starting QEMU..."
        qemu-system-i386 -fda ./build/ExploreOS-0.45.0028.img
    elif command -v qemu-system-x86_64 &> /dev/null; then
        qemu-system-x86_64 -fda ./build/TS-1.00.0001.img
    else
        echo "QEMU is not installed. Please install it to test ExploreOS."
    fi
fi
