#!/bin/bash


if ! command -v nasm &> /dev/null; then
    echo "Error: NASM (Netwide Assembler) is not installed or not in your PATH."
    echo "Please install NASM before running this script."
    exit 1
fi
mkdir -p build
nasm -f bin ./src/boot/boot.asm -o ./build/boot.bin 
nasm -f bin ./src/kernel/kernel.asm -o ./build/kernel.bin 

dd if=/dev/zero of=./build/TS-1.00.0001.img bs=1024 count=1440
dd if=./build/boot.bin of=./build/TS-1.00.0001.img conv=notrunc
dd if=./build/kernel.bin of=./build/TS-1.00.0001.img bs=512 seek=1 conv=notrunc

rm ./build/*.bin
echo "Build complete! Disk image created at ./build/TS-1.00.0001.img"
echo "Would you like to test it in QEMU? (y/n)"
read answer
if [ "$answer" = "y" ] || [ "$answer" = "Y" ]; then
    if command -v qemu-system-i386 &> /dev/null; then
        echo "Starting QEMU..."
        qemu-system-i386 -fda ./build/TS-1.00.0001.img
    elif command -v qemu-system-x86_64 &> /dev/null; then 
        qemu-system-x86_64 -fda ./build/TS-1.00.0001.img
    else 
        echo "QEMU is not installed. Please install it to test ExploreOS."
    fi
fi
