; boot.asm

[BITS 16]
[ORG 0x7C00]
KERNEL_OFFSET equ 0x1000

start:
    ; Standard initialization
    xor ax, ax
    mov ds, ax
    mov es, ax
    mov ss, ax
    mov sp, 0x7C00
    
    ; Save boot drive
    mov [boot_drive], dl
    
    ; Check for USB support
    mov ax, 0x4100
    mov bx, 0x55AA
    int 0x13
    jc legacy_boot
    cmp bx, 0xAA55
    jne legacy_boot
    test cl, 1
    jz legacy_boot
    
    ; USB boot attempt
    mov si, usb_msg
    call print_string
    mov ah, 0x42
    mov dl, [boot_drive]
    mov si, dap
    int 0x13
    jnc jump_kernel
    
legacy_boot:
    ; Traditional disk read
    mov ah, 0x02
    mov al, 2
    mov ch, 0
    mov cl, 2
    mov dh, 0
    mov dl, [boot_drive]
    mov bx, KERNEL_OFFSET
    int 0x13
    jc disk_error

jump_kernel:
    jmp KERNEL_OFFSET

disk_error:
    mov si, error_msg
    call print_string
    jmp $

print_string:
    mov ah, 0x0E
.loop:
    lodsb
    test al, al
    jz .done
    int 0x10
    jmp .loop
.done:
    ret

; Disk Address Packet for USB boot
dap:
    db 0x10      ; Size of DAP
    db 0         ; Reserved
    dw 2         ; Number of sectors
    dw KERNEL_OFFSET  ; Offset
    dw 0         ; Segment
    dq 1         ; Starting LBA

boot_drive db 0
error_msg db 'Error loading Typing Simulator.', 0
usb_msg db 'Attempting USB boot...', 0
times 510-($-$$) db 0
dw 0xAA55