[BITS 16]
[ORG 0x1000]
start:
    mov ax, 0x0003
    int 0x10
    mov si, welcome_msg
    call print_string
    ; Initialize cursor position tracking
    mov word [cursor_x], 0
    mov word [cursor_y], 4  ; Account for welcome message lines
main_loop:
    ; Get current cursor position first
    mov ah, 0x03      ; Get cursor position function
    mov bh, 0         ; Page number
    int 0x10
    mov [cursor_y], dh ; Row
    mov [cursor_x], dl ; Column
    
    mov ah, 0x00
    int 0x16
    cmp ah, 0x3F      ; F5 scan code
    je clear_screen
    ; Check if character is ASCII (0x20 to 0x7E)
    cmp al, 0x20
    jb check_control
    cmp al, 0x7E
    ja main_loop
    mov ah, 0x0E
    int 0x10
    jmp main_loop
check_control:
    cmp al, 0x0D      ; Enter
    je handle_enter
    cmp al, 0x08      ; Backspace  
    je handle_backspace
    jmp main_loop
clear_screen:
    mov ax, 0x0003    ; Text mode 80x25 16 colors
    int 0x10
    jmp main_loop
handle_enter:
    mov ah, 0x0E
    mov al, 0x0D
    int 0x10
    mov al, 0x0A
    int 0x10
    jmp main_loop
handle_backspace:
    ; Get current cursor position
    mov ah, 0x03      ; Get cursor position
    mov bh, 0         ; Page number
    int 0x10
    
    ; Check if we're at the start of a line
    cmp dl, 0
    jne standard_backspace
    
    ; We're at column 0, check if we're on the first line
    cmp dh, 0
    je main_loop      ; If at top left, do nothing
    
    ; Find end of previous line (by reading attributes to see last non-space char)
    ; This is an optimization we could add, but for now we'll just go to column 79
    
    ; Move cursor to end of previous line
    dec dh            ; Move up one row
    mov dl, 79        ; Move to last column
    
    ; Set new cursor position
    mov ah, 0x02      ; Set cursor position
    mov bh, 0         ; Page number 
    int 0x10
    
    ; Overwrite with space
    mov ah, 0x0E      ; Teletype output
    mov al, ' '
    int 0x10
    
    ; Reposition cursor (since printing space moves it)
    mov ah, 0x02
    mov dh, dh        ; Keep current row
    mov dl, 79        ; Last column
    int 0x10
    
    jmp main_loop
standard_backspace:
    mov ah, 0x0E
    mov al, 0x08      ; Backspace
    int 0x10
    mov al, ' '       ; Space (to erase)
    int 0x10
    mov al, 0x08      ; Backspace again (to position cursor)
    int 0x10
    jmp main_loop
print_string:
    mov ah, 0x0E
.loop:
    lodsb
    test al, al
    jz .done
    int 0x10
    jmp .loop
.done:
    ret

; Data section
cursor_x dw 0
cursor_y dw 0
welcome_msg db 'ExploreSoft (C) Typing Simulator (TM) Version 1.0.0', 0x0D, 0x0A
            db 'Press F5 to clear the screen.', 0x0D, 0x0A, 0x0D, 0x0A
            db 'Start typing...', 0x0D, 0x0A, 0x0D, 0x0A, 0
times 1024-($-$$) db 0
